/**
 * Copyright (c) 2016 Red Hat Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Kurtakov - initial API and implementation.
 */
package org.fedoraproject.abrt;

import static org.eclipse.epp.logging.aeri.core.ProblemStatus.UNCONFIRMED;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.management.ManagementFactory;
import java.nio.channels.Channels;
import java.nio.charset.StandardCharsets;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.epp.logging.aeri.core.IModelFactory;
import org.eclipse.epp.logging.aeri.core.IProblemState;
import org.eclipse.epp.logging.aeri.core.IReport;
import org.eclipse.epp.logging.aeri.core.ISendOptions;
import org.eclipse.epp.logging.aeri.core.IServerConnection;
import org.eclipse.epp.logging.aeri.core.ProblemStatus;
import org.eclipse.epp.logging.aeri.core.util.Reports;
import org.osgi.framework.Bundle;

import jnr.unixsocket.UnixSocketAddress;
import jnr.unixsocket.UnixSocketChannel;

public class AbrtConnection implements IServerConnection {

    @Override
    public IProblemState interested(IStatus status, IEclipseContext context, IProgressMonitor monitor) {
        IProblemState state = IModelFactory.eINSTANCE.createProblemState();
        // always interested.
        state.setStatus(UNCONFIRMED);
        return state;
    }

    @Override
    public IReport transform(IStatus status, IEclipseContext context) {
        // If your in-house app allows some kind of send options, this is the right time to apply those options:

        // if it's just fire-and-forget, this is not needed:
        ISendOptions options = context.get(ISendOptions.class);
        IReport report = Reports.newReport(status);
        report.setComment(options.getComment());
        report.setAnonymousId(options.getReporterId());
        report.setName(options.getReporterName());
        report.setEmail(options.getReporterEmail());
        report.setSeverity(options.getSeverity());

        if (options.isAnonymizeMessages()) {
            Reports.anonymizeMessages(report);
        }
        if (options.isAnonymizeStackTraces()) {
            // Reports.anonymizeStackTraces(report, Pattern.compile("org\\.eclipse\\..*"));
        }
        // in case additional analyzers will be executed (e.g., class-path analysis for class loading issues etc.), you can compute them
        // here once and put the result into the context passed to this method (to remember the computation result)
        return report;
    }

    @Override
    public IProblemState submit(IStatus status, IEclipseContext context, IProgressMonitor monitor) throws IOException {
        // here the upload takes place. This can by anything from an HTTP POST to some server, just dumping a file on a shared network
        // device - whatever is needed.
        upload(status);

        // In our example here we'll just return a dummy state. With the in-house work-flow used, no-one will ever see this, so...
        IProblemState response = IModelFactory.eINSTANCE.createProblemState();
        response.setStatus(ProblemStatus.CONFIRMED);
        return response;
    }

    private static void upload(IStatus status) {
        File path = new File("/var/run/abrt/abrt.socket");
        UnixSocketAddress address = new UnixSocketAddress(path);
        try (UnixSocketChannel channel = UnixSocketChannel.open(address); OutputStream w = Channels.newOutputStream(channel)) {
            w.write(getBytesOutput("POST / HTTP/1.1\r\n"));
            w.write(getBytesOutput("\r\n"));
            w.write(getBytesOutput("type=libreport\0"));
            String processName = ManagementFactory.getRuntimeMXBean().getName();
            w.write(getBytesOutput("pid=" + processName.substring(0, processName.indexOf('@')) + "\0"));
            w.write(getBytesOutput("reason=" + status.getMessage() + "\0"));
            appendComponentExecutable(w, status.getPlugin());
            w.write(getBytesOutput("eclipsePlugin=" + status.getPlugin() + "\0"));
            w.write(getBytesOutput("duphash=" + status.hashCode() + "\0"));
            w.write(getBytesOutput("uuid=" + status.hashCode() + "\0"));
            StringWriter writer = new StringWriter();
            PrintWriter pWriter = new PrintWriter(writer);
            status.getException().printStackTrace(pWriter);
            w.write(getBytesOutput("backtrace=" + writer.toString() + "\0"));
            w.write(getBytesOutput("\0"));
            w.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void appendComponentExecutable(OutputStream w, String pluginId) throws IOException {
        Bundle bundle = Platform.getBundle(pluginId);
        if (bundle != null) {
            File bundlefile = FileLocator.getBundleFile(bundle);
            if (bundlefile.getAbsolutePath().startsWith("/usr")) {
                w.write(getBytesOutput("executable=" + bundlefile.getAbsolutePath() + "\0"));
            }

        } else {
	        w.write(getBytesOutput("executable=/usr/bin/eclipse\0"));
	}
    }

    private static byte[] getBytesOutput(String content) {
        return content.getBytes(StandardCharsets.US_ASCII);
    }

    @Override
    public void discarded(IStatus status, IEclipseContext context) {
    }

}
